Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;
-------------------------------------------------------------------- 

entity Flash_LED is
	port(clk      : in std_logic;
		 reset_n  : in std_logic;
		 Input	  : in std_logic;
		 led_out  : out std_logic
		);
end Flash_LED;
	
architecture main of Flash_LED is
	
	-- Counter/Timer declarations --------------------------------------
	signal nclr_cnt : std_logic;
	signal got_cnt_flag: std_logic;
	signal time_mark : std_logic_vector(31 downto 0);

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(rdy_off, led_off, rdy_on, led_on);
	signal CS, NS  : STATE_TYPE;

begin -- main --------------------------------------------------------
		
	-------------------------------------------------------------------
	SUMCNT: counter  -- Timer
	generic map(WID32)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => time_mark,
			   flag_out => got_cnt_flag,
			   cnt_out  => open
			 );
	
-- State Machine define initialization ---------------------------
	sm_reg: process (clk, reset_n)	
	begin
		if(reset_n = '0')        then CS <= rdy_off;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, got_cnt_flag, Input)
	begin
		-- default signal values
				 time_mark <= X"00811C40";
--				 time_mark <= X"01023880";
				--time_mark <= X"00000004"; -- short delay for simulation
				nclr_cnt <= '1';
				led_out <= '0';
		-------------------------
		case CS is
			when rdy_off   =>
				nclr_cnt <= '0';
				if(Input = '1') then 		NS <= led_off;
				else						NS <= rdy_off;
				end if;
			    
			when led_off   =>
				if(got_cnt_flag = '1') then NS <= rdy_on;
				else                      	NS <= led_off;
				end if;
			    
			when rdy_on  =>
				nclr_cnt <= '0';
				if(Input = '0') then		NS <= led_on;
				else						NS <= rdy_on;
				end if;
			    
			when led_on  =>
				if(got_cnt_flag = '1') then NS <= rdy_off;
				else                      	NS <= led_on;
				end if;
				led_out <= '1';
						
		end case; 
				
	end process; -- sm_combinatorial

end main;

