Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, March 17, 2010
--
-- The memory space open to the Lower FPGA is from 0x20000 to 0x7FFFF.
-- The Upper FPGA VME interface logic will set the bit 19 of the address
-- to '1' for any VME Write and VME Reads of the Upper FPGA or Flash.  
-- This will set the address greater than 0x7FFFF.  Bit 19 is cleared
-- before sending the address on for Lower FPGA memory map decoding.		

entity Lower_VME_Interface is 
	port( 
		Upper_Addr_To_Mem_in : in std_logic_vector(19 downto 0);
		VME_Data_In			 	: in std_logic_vector(31 downto 16);
		Upper_Data_To_Mem_in : in std_logic_vector(31 downto 0);
		Lower_Data_To_Interface : in std_logic_vector(31 downto 0);
		
		Upper_Addr_To_Mem	: out std_logic_vector(19 downto 0);
		VME_Data_Out		: out std_logic_vector(31 downto 16);
		Data_To_Mem			: out std_logic_vector(31 downto 0);
		InterFPGA_D_Out 	: out std_logic_vector(31 downto 0)			
		);
	end Lower_VME_Interface;
	
architecture main of Lower_VME_Interface is	

signal VME_Process_Active 	 : std_logic;

begin

	VME_Process_Active <= Upper_Addr_To_Mem_in(19);
	
	-- clear extra bit for VME Writes to Lower FPGA Memory
	Upper_Addr_To_Mem(19) 			<= '0'; 		
	Upper_Addr_To_Mem(18 downto 0) 	<= Upper_Addr_To_Mem_in(18 downto 0); 	

------------------------------------------------------------------------------------		
	process (VME_Data_In, Upper_Data_To_Mem_in,
				Lower_Data_To_Interface, VME_Process_Active)
	begin
		-------------------------------------------------------------------------------	
		-- Note VME_Process_Active is not set for VME Reads of the Lower FPGA	 
		if   (VME_Process_Active = '0') then 
		
				-- This sets up for an Upper FPGA Write to the Lower FPGA or SRAM memory
				Data_To_Mem(31 downto 16) 		<= Upper_Data_To_Mem_in(31 downto 16);
				Data_To_Mem(15 downto  0) 		<= Upper_Data_To_Mem_in(15 downto 0);
				
				-- This sets up for an Upper FPGA Read of the Lower FPGA or SRAM memory
				InterFPGA_D_Out(31 downto 16) 	<= Lower_Data_To_Interface(31 downto 16);
				InterFPGA_D_Out(15 downto  0) 	<= Lower_Data_To_Interface(15 downto 0);
				
				-- This sets up for a VME Read of the Lower FPGA
				VME_Data_Out(31 downto 16) 		<= Lower_Data_To_Interface(31 downto 16);
		-------------------------------------------------------------------------------		
		else	-- VME Write Operation or VME Read of Upper FPGA
		
				-- This sets up for a VME Write to Lower FPGA or SRAM memory
				Data_To_Mem(31 downto 16) 		<= VME_Data_In(31 downto 16);
				Data_To_Mem(15 downto  0) 		<= Upper_Data_To_Mem_in(15 downto 0);
				
				-- This sets up for a VME Write to Upper FPGA or Flash memory
				InterFPGA_D_Out(31 downto 16) 	<= VME_Data_In(31 downto 16);  
				InterFPGA_D_Out(15 downto  0) 	<= Lower_Data_To_Interface(15 downto 0);
				
				-- This sets up for a VME Read of the Upper FPGA.
				VME_Data_Out(31 downto 16) 		<= Upper_Data_To_Mem_in(31 downto 16);
		end if;
		
	end process;

end;