--------------------------------------------------------------------
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my defpower_up_stateions and entities

-------------------------------------------------------------------- 

entity Startup_Reset is
	port(clk      		: in std_logic;
		 start			: in std_logic;
		 ext_reset_n  	: in std_logic;
		 ena_2p5V 		: out std_logic;
		 ena_5p0V 		: out std_logic;
		 ena_inputs		: out std_logic;
		 reset_pll		: out std_logic;
		 reset_out_n 	: out std_logic
		);
end Startup_Reset;
	
architecture main of Startup_Reset is
	
-- Counter/Timer declarations --------------------------------------
signal nclr_cnt 	: std_logic;
signal got_cnt_flag	: std_logic;
signal time_mark 	: std_logic_vector(19 downto 0);

-- State Machine declarations ------------------------------------
type STATE_TYPE is 
(power_up_state, pause, seq_power, power_wait, seq_power2, power_wait2, seq_done);
signal CS, NS  : STATE_TYPE;
	

begin -- main --------------------------------------------------------
		
	-------------------------------------------------------------------
	PAUSE_COUNTER: counter  -- Timer
	generic map(20)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => time_mark,
			   flag_out => got_cnt_flag,
			   cnt_out  => open
			 );
	
-- State Machine define initialization ---------------------------
	sm_reg: process (clk, start)	
	begin
		if   (start = '0')      	then CS <= power_up_state;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, got_cnt_flag, ext_reset_n)
	begin
		-- default signal values
				reset_out_n <= '0';
				reset_pll	<= '0';
				time_mark 	<= X"00004"; -- short delay
				nclr_cnt 	<= '0'; -- pause timer normally in reset
				ena_2p5V 	<= '0';
				ena_5p0V 	<= '0';
				ena_inputs  <= '0';
		-------------------------
		case CS is
			when power_up_state   =>		NS <= pause;
				reset_pll	<= '1';
			    ena_2p5V 	<= '0';
			    ena_5p0V 	<= '0';
			    
			when pause   => -- sit briefly with reset active before sequence
				reset_pll	<= '1';
				nclr_cnt 	<= '1';
				time_mark 	<= X"00004";
				if(got_cnt_flag = '1') then NS <= seq_power;
				else                      	NS <= pause;
				end if;
			    
			when seq_power  =>				NS <= power_wait;
			    ena_2p5V 	<= '1'; -- Let the 2.5V come on first
				nclr_cnt 	<= '0';	-- Clear the pause counter also			
			    
			when power_wait  =>
			    ena_2p5V 	<= '1';
				nclr_cnt 	<= '1';			
				time_mark 	<= X"13880";
				if(got_cnt_flag = '1') then NS <= seq_power2;
				else                      	NS <= power_wait;
				end if;
				
			when seq_power2  =>				NS <= power_wait2;
			    ena_2p5V 	<= '1';
				ena_5p0V 	<= '1'; -- Let the 5.0V come on second
				nclr_cnt 	<= '0';	-- Clear the pause counter also			
			    
			when power_wait2  =>
			    ena_2p5V 	<= '1';
				ena_5p0V 	<= '1';
				nclr_cnt 	<= '1';			
				time_mark 	<= X"03A98";
				if(got_cnt_flag = '1') then NS <= seq_done;
				else                      	NS <= power_wait2;
				end if;
			
			when seq_done  =>				NS <= seq_done;
				reset_out_n <= ext_reset_n; -- has been force active until now
				ena_2p5V 	<= '1';
				ena_5p0V 	<= '1';
				ena_inputs	<= '1'; -- lastly we enable analog inputs
		
		end case; 
				
	end process; -- sm_combinatorial

end main;

