Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, 
-- The conversion of the ADC code output is a bit tricky.
--
-- Below is the conversions that we do
--
--                         sign extended to 16 bit
-- volts in   | 15 bit OB | 15 bit 2's | Phase Angle
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
--   +10 V    |   0x7FFF  |   0x3FFF   |  +90 deg
--    +5 V	  |   0x6000  |   0x2000	|  +45 deg
--    +0 V    |   0x4000  |   0x0000   |    0 deg
--    -0 V    |   0x3FFF  |   0xFFFF   |    0 deg
--    -5 V    |   0x2000  |   0xE000   |  -45 deg
--   -10 V    |   0x0000  |   0xC000   |  -90 deg

-- The full range for the phase shift is -180 to +180,
-- given a code of 0x8000 to 0x7FFF.

entity ADC_Limiter_v1 is 
	port( 
			clk		: in std_logic;
			nreset	: in std_logic;
			ADC_in	: in std_logic_vector(15 downto 0);
			ADC_Strb : in std_logic;
			
			ADC_out			: out std_logic_vector(15 downto 0);
			ADC_Strb_out	: out std_logic					
		);
	end ADC_Limiter_v1;
	
architecture main of ADC_Limiter_v1 is	

signal ADC_limited	: std_logic_vector(15 downto 0);

begin	
	
	process(clk, nreset, ADC_in)
	begin
		if   (nreset = '0') then ADC_limited <= (others => '0');
		elsif rising_edge(clk) then
			if   (ADC_in(15) = '1') then
					ADC_limited(15 downto 0) <= not ADC_in(15 downto 0);
			else	ADC_limited(15 downto 0) <= ADC_in(15 downto 0);			
			end if;
		end if;
	end process;
	
	ADC_out(15) <= not ADC_limited(14);
	ADC_out(14) <= not ADC_limited(14);
	
	ADC_out(13 downto 0) <= ADC_limited(13 downto 0);
	
	ADC_Strb_out <= ADC_Strb;
	
end;