Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

----------------------------------------------------------------
-- Scales and offsets the 32 bit frequency word down to 16 bits.

-- Basic frequency word scale factor is 0.11175869 Hz/Bit.
-- X"1428F5FB" = 338228732 bits -> 37.800000 MHz
-- We want 37.8 MHz -> 0x08000000
-- 0x1428F5FB - 0x08000000 - 0x08000000 = 0x0428F5FB

-- 53 MHz minus 37.8 MHz is 15.2 MHz -> 1360007320 bits (X"81B4E98")

-- dividing the value shifting 12 times gives a number with a 
-- resolution of 2^12*0.11175869 = 457 Hz.

-- after shifting the 15.2 MHz range -> 0x81B4 (fits in 16 bits)

entity DDS_FREQ_to_DAC_v2 is 
	port( 	
			clk					: in std_logic;
			nreset				: in std_logic;
			DDS_Frequency		: in std_logic_vector(31 downto 0);
			DDS_Update_Strobe	: in std_logic;
			
			DAC_Data_Out		: out std_logic_vector(15 downto 0)			
		);
	end DDS_FREQ_to_DAC_v2;
	
architecture main of DDS_FREQ_to_DAC_v2 is	

	component add_sub32
		PORT
		(
			add_sub		: IN STD_LOGIC ;
			dataa			: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
			datab			: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
			result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
		);
	end component;


	signal data_buffer			: std_logic_vector(15 downto 0);
	signal subtraction_result 	: std_logic_vector(31 downto 0);
	
--======================================================================
--               B  E  G  I  N  
------------------------------------------------------------------------       
begin
SUBTRACT_OFFSET: 
	add_sub32 port map
					(	add_sub	=>	'0', --SUBTRACT
						dataa		=>	DDS_Frequency,
--						datab		=> X"0428F5FB",
						datab		=> X"042E4921", -- 0.5% more
						result	=> subtraction_result 
					  );

DATA_LATCH:
process(clk, nreset, DDS_Update_Strobe, subtraction_result)
begin
	if   (nreset = '0') 		then data_buffer <= (others => '0');
	elsif rising_edge(clk) 	then
		if   (DDS_Update_Strobe = '1') then data_buffer <= subtraction_result(27 downto 12);
		end if;
	end if;
end process;

--DAC_Data_Out <= data_buffer; -- DATA OUTPUT ----------
--DAC_Data_Out <= not data_buffer(15) & data_buffer(14 downto 0); -- DATA OUTPUT --- Run through Daig Mux which expects 2's complement-------
DAC_Data_Out <= data_buffer(15 downto 0); -- DATA OUTPUT --- Bit 15 get's flipped at a later stage -------


end;