Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------
-- MODULE DESCRIPTION
-- The module performs the following functions
-- 	1. Latches the frequency and phase data words and 
--		resyncronizes the data and the update strobe
--		with respect to the DDS SCLK (68 MHz)
--
--	2. It packs the Frequency and Phase word to be written
-- 		to the Profile 0 register of the DDS.  It also
--		reverses the bits of the word for loading the DDS
--		register LSB First.
------------------------------------------------------

entity Freq_Phase_Update_Setup is
	port( 	
			SCLK			: in std_logic;
			Reset_n			: in std_logic;
			Freq_Data		: in std_logic_vector(31 downto 0);
			Phase_Data		: in std_logic_vector(15 downto 0);
			Update_Strobe	: in std_logic;
				
			Freq_Phase_Data	: out std_logic_vector(71 downto 0);
			Update_TxStrb	: out std_logic;
			BitCount        : out std_logic_vector(6 downto 0);
			MSB_nLSB		: out std_logic;
			Ena_IOReset		: out std_logic;
			CS				: out std_logic
		);
end Freq_Phase_Update_Setup;

architecture main of Freq_Phase_Update_Setup  is

	signal sync_strb	: std_logic_vector(3 downto 0);
	signal get_data		: std_logic;
	signal Data 		: std_logic_vector(71 downto 0);
	
begin

	process (SCLK, Reset_n, Update_Strobe, sync_strb)
	begin
		if   (Reset_n = '0') then sync_strb <= (others => '0');
		elsif rising_edge(SCLK) then
			sync_strb(0) 	<= Update_Strobe;
			sync_strb(1) 	<= sync_strb(0);
			sync_strb(2) 	<= sync_strb(1);
			sync_strb(3) 	<= sync_strb(2);
		end if;
		get_data 		<= sync_strb(1) AND NOT sync_strb(2); -- LED pulse out
		Update_TxStrb 	<= sync_strb(2) AND NOT sync_strb(3); -- transmit strobe out
	end process;
	
	process (SCLK, Reset_n, get_data, Freq_Data, Phase_Data)
	begin
		if   (Reset_n = '0') then Data <= (others => '0');
		elsif rising_edge(SCLK) then
			if (get_data = '1') then
				Data( 7 downto  0) <= "00001110"; -- instruction byte => PROFILE 0
				Data(39 downto  8) <= Freq_Data(31 downto 0);
				Data(55 downto 40) <= Phase_Data(15 downto 0);
				Data(71 downto 56) <= (others => '0');
			end if;		
		end if;
	end process;
	
	BitCount        <= "0110111"; -- 56 bits to be sent (count = 55)
	MSB_nLSB		<= '0';
	Ena_IOReset		<= '1';
	CS				<= '1';

-- Reverse the bits of the 72 bit word --
	process ( Data )
		variable dd : std_logic_vector(71 downto 0);
	begin
		for i in 71 downto 0 loop dd(i) := Data(71-i);
		end loop;
		Freq_Phase_Data <= dd;
	end process;

end main;