Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 
USE work.DDS_VME_CONSTANTS.all;

----------------------------------------------------------------

entity Upper_IO_Regs is 
	port( 	
			clk			: in std_logic;
			nreset		: in std_logic;
			Sample_Cmd	: in std_logic;			
			Addr_to_Mem	: in std_logic_vector(23 downto 0);
			
			ADC1		: in std_logic_vector(15 downto 0);
			ADC1_STRB	: in std_logic;	
			
			ADC2		: in std_logic_vector(15 downto 0);
			ADC2_STRB	: in std_logic;	
						
			IO_Data_Out	: out std_logic_vector(31 downto 0);
			IO_reg_1	: out std_logic_vector(15 downto 0);
			IO_reg_2	: out std_logic_vector(15 downto 0)
		);
	end Upper_IO_Regs;
	
architecture main of Upper_IO_Regs is	

signal sync_sample  : std_logic_vector(2 downto 0);
signal sample		: std_logic;

signal ADDR 		: std_logic_vector(23 downto 0);

signal reg_1_out	: std_logic_vector(15 downto 0);
signal reg_2_out	: std_logic_vector(15 downto 0);

signal adc1_sample	: std_logic;
signal adc2_sample	: std_logic;
           
begin
	--------------------------------------------------------------
	ADDR <= Addr_to_Mem;
	--------------------------------------------------------------
	SYNCRONIZE_SAMPLE:
	process(clk, nreset, Sample_Cmd)
    begin
		if   (nreset = '0') then sync_sample <= "000";
        elsif rising_edge(clk) then
          sync_sample(0) <= Sample_Cmd;
          sync_sample(1) <= sync_sample(0);
          sync_sample(2) <= sync_sample(1);
        end if;
    end process;
    
    sample <= sync_sample(1) AND NOT sync_sample(2);
    
    LATCH_SAMPLE_ADC1:
    process(clk, nreset, sample, ADC1_STRB)
    begin
		if   (nreset = '0') 		then adc1_sample <= '0';         
        elsif rising_edge(clk) 		then
			if   (sample = '1'   ) 	then adc1_sample <= '1';
			elsif(ADC1_STRB = '1') 	then adc1_sample <= '0';
			end if;
		end if;
	end process;		-- adc1_sample goes high and stays high until
						-- the following strobe latches the new data.

    LATCH_SAMPLE_ADC2:
    process(clk, nreset, sample, ADC2_STRB)
    begin
		if   (nreset = '0') 		then adc2_sample <= '0';         
        elsif rising_edge(clk) 		then
			if   (sample = '1'   ) 	then adc2_sample <= '1';
			elsif(ADC2_STRB = '1') 	then adc2_sample <= '0';
			end if;
		end if;
	end process;    

    --------------------------------------------------------------
	
	PARAMETER_REGISTERS: 
	process(clk, nreset, ADC1_STRB, ADC2_STRB)
	begin	
		if rising_edge(clk) then
			if (nreset = '0') then
				reg_1_out <= (others => '0');
				reg_2_out <= (others => '0');
			else
				if (adc1_sample = '1')	then	
					reg_1_out <= ADC1; 
					end if;
					
				if (adc2_sample = '1')	then	
					reg_2_out <= ADC2; 
					end if;
			end if;                                            
		end if;
	end process;
       
	DATA_OUT_MUX: 
	process(ADDR, reg_1_out, reg_2_out)
	begin	
		if   (ADDR = IO_UPPER_REG_SPACE     ) then 	IO_Data_Out <= X"0000" & reg_1_out;
			
		elsif(ADDR = IO_UPPER_REG_SPACE + 4 ) then	IO_Data_Out <= X"0000" & reg_2_out;
		else														IO_Data_Out <= (others => '0');			
		end if;                                 				
	end process;

	IO_reg_1 <= reg_1_out;
	IO_reg_2 <= reg_2_out;
          
end;